import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.util.Vector;

public class JS8Level extends DefaultHandler {
	//Vars of the JS8Level
	private int[][][] tiles;
	private boolean[][][] apples;
	private boolean[][][] snake;
	
	private Vector<Function> funcs;
	private Vector<Block> blocks;
	private String funcNewApples;
	private String funcApplesTillNew;
	private int newApples;
	private int applesTillNew;
	private int applesThisTurn;
	private boolean blind;
	private int neededApples;
	private String author;
	private String name;
	
	public JS8Level(String file) {
		tiles = new int[7][16][25];
		apples = new boolean[7][16][25];
		snake = new boolean[7][16][25];
		
		init(file);
	}
	
	public int getTile(Coord d) {
		Coord c = d.add(new Coord(0,0,1));
		if (c.getX() < 0) return 0;
		if (c.getY() < 0) return 0;
		if (c.getX() > 24) return 0;
		if (c.getY() > 15) return 0;
		if (getSnake(c)) return 0;
		return tiles[c.getZ()][c.getY()][c.getX()];
	}
	
	public boolean isAppleAt(Coord d) {
		Coord c = d.add(new Coord(0,0,1));
		return apples[c.getZ()][c.getY()][c.getX()];
	}
	
	public void removeApple(Coord d) {
		Coord c = d.add(new Coord(0,0,1));
		apples[c.getZ()][c.getY()][c.getX()] = false;
		applesThisTurn++;
		if (applesThisTurn == applesTillNew) {
			calcNewApples();
			calcApplesTillNew();
			placeApples();
		}
	}
	public int getMinApples() {
		return neededApples;
	}

	public boolean timerUsed() {
		return false;
	}
	
	public void setSnake(Coord d, boolean s) {
		if (d == null) return;
		Coord c = d.add(new Coord(0,0,1));
		if (c.getX()<0) return;
		if (c.getY()<0) return;
		if (c.getX()>24) return;
		if (c.getY()>15) return;
		snake[c.getZ()][c.getY()][c.getX()] = s;
	}
	
	private boolean getSnake(Coord c) {
		return snake[c.getZ()][c.getY()][c.getX()];
	}
	
	public Coord getStartPos() {
		Coord c;
		c = new Coord((int) (Math.random()*25),
		              (int) (Math.random()*16),
		              (int) (Math.random()*5));
		while( (getTile(c)%6 != 1) || isAppleAt(c) )
			c = new Coord((int) (Math.random()*25),
			              (int) (Math.random()*16),
			              (int) (Math.random()*5));
		return c;
	}
	
	public void init(String file) {
		for (Coord c : Coord.range(new Coord(0,0,0), new Coord(24,15,6))) {
			tiles[c.getZ()][c.getY()][c.getX()] = 0;
			apples[c.getZ()][c.getY()][c.getX()] = false;
			snake[c.getZ()][c.getY()][c.getX()] = false;
		}
		blocks = new Vector<Block>();
		funcs = new Vector<Function>();
		funcs.add(new Function("const", "add", "prev", "0", funcs));
		funcNewApples = "const";
		funcApplesTillNew = "const";
		newApples = 1;
		applesTillNew = 1;
		blind = false;
		neededApples = 0;
		author = "Unknown";
		name = "No name";
		
		try {
			DefaultHandler handler = this;
			SAXParser saxParser = SAXParserFactory.newInstance().newSAXParser();
			saxParser.parse( new File( file ), handler );
		} catch( Throwable t ) {
			t.printStackTrace();
		}
		
		placeApples();
	}
	private void placeApples() {
		Coord d;
		for(int i = 0; i < newApples; i++) {
			d = new Coord((int) (Math.random()*25),
			              (int) (Math.random()*16),
			              (int) (Math.random()*5));
			while(isAppleAt(d) || (getTile(d)%6 != 1))
				d = new Coord((int) (Math.random()*25),
				              (int) (Math.random()*16),
				              (int) (Math.random()*5));
			Coord c = d.add(new Coord(0,0,1));
			apples[c.getZ()][c.getY()][c.getX()] = true;
		}
		applesThisTurn = 0;
	}
	
	private void calcNewApples() {
		//search used function
		Function f = search(funcNewApples);
		if (f != null) newApples = f.calc(newApples);
	}
	
	private void calcApplesTillNew() {
		//search used function
		Function f = search(funcApplesTillNew);
		if (f!=null) applesTillNew = f.calc(applesTillNew);
	}
	
	private Function search(String func) {
		Function f = null;
		for (Function g : funcs) if (g.getName().equals(func)) f = g;
		return f;
	}
	
	//ode for parsing levelfile
	public void startDocument() throws SAXException {
	}
	public void endDocument() throws SAXException {
	}
	public void characters( char[] buf, int offset, int len ) throws SAXException {
	}
	
	public void startElement( String namespaceURI,
	                          String localName,   // local name
	                          String qName,       // qualified name
	                          Attributes attr )
	                         throws SAXException {
		if (qName.equals("info")) {
			parseInfo(attr);
		} else if (qName.equals("func")) {
			funcs.add(new Function(attr.getValue("name"), attr.getValue("type"), attr.getValue("val1"), attr.getValue("val2"), funcs));
		} else if (qName.equals("block")) {
			parseBlock(attr);
		} else if (qName.equals("quad")) {
			parseQuad(attr);
		} else if (qName.equals("cell")) {
			int f = Integer.parseInt(attr.getValue("fill"));
			int z = Integer.parseInt(attr.getValue("z"))+1;
			int y = Integer.parseInt(attr.getValue("y"));
			int x = Integer.parseInt(attr.getValue("x"));
			tiles[z][y][x] = f;
		}
	}
	
	public void endElement( String namespaceURI,
	                        String localName,     // local name
	                        String qName )        // qualified name
	                       throws SAXException {
	}
	
	private void parseInfo(Attributes attr) {
		if (attr.getValue("type").equals("author"))
			author = attr.getValue("content");
		if (attr.getValue("type").equals("name"))
			name = attr.getValue("content");
		if (attr.getValue("type").equals("startApplesNew"))
			newApples = Integer.parseInt(attr.getValue("content"));
		if (attr.getValue("type").equals("funcApplesNew"))
			funcNewApples = attr.getValue("content");
		if (attr.getValue("type").equals("startApplesTillNew"))
			applesTillNew = Integer.parseInt(attr.getValue("content"));
		if (attr.getValue("type").equals("funcApplesTillNew"))
			funcApplesTillNew = attr.getValue("content");
		if (attr.getValue("type").equals("neededApples"))
			neededApples = Integer.parseInt(attr.getValue("content"));
		if (attr.getValue("type").equals("blind")) {
			if (attr.getValue("content").equals("true")) blind=true;
			else blind = false;
		}
	}
	
	private void parseBlock(Attributes attr) {
		Block b = new Block();
		b.setPos(Integer.parseInt(attr.getValue("sx")),
		         Integer.parseInt(attr.getValue("sy")),
		         Integer.parseInt(attr.getValue("sz")));
		b.setDir(Integer.parseInt(attr.getValue("mx")),
		         Integer.parseInt(attr.getValue("my")));
		blocks.add(b);
	}
	
	private void parseQuad(Attributes attr) {
		int f = Integer.parseInt(attr.getValue("fill"));
		for (Coord c : Coord.range( new Coord(Integer.parseInt(attr.getValue("sx")), 
		                                      Integer.parseInt(attr.getValue("sy")),
		                                      Integer.parseInt(attr.getValue("sz"))+1),
		                            new Coord(Integer.parseInt(attr.getValue("ex")),
		                                      Integer.parseInt(attr.getValue("ey")),
		                                      Integer.parseInt(attr.getValue("ez"))+1) ))
			tiles[c.getZ()][c.getY()][c.getX()] = f;
	}
}
