/*(c) JippySnake Entwicklungsteam.
 *
 *
 *Dieser Code wird unter der GNU Public Licence vertireben
 *
 *
 */

/* Modified for Shopping-ManaJer */

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.BufferStrategy;
import java.util.HashMap;
import javax.imageio.ImageIO;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.RenderingHints;
import java.awt.RenderingHints.Key;
import java.net.URL;
import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
/*
	The drawing layers:
	0: Backgrund layer (off = 0)
	1: Snake layer 0 (off=0)
	2: Level layer 0 (off=-1)
	3: Snake layer 1 (off=-1)
	4: Level layer 1 (off=-2)
	5: Snake layer 2 (off=-2)
	6: Level layer 2 (off=-3)
	7: Snake layer 3 (off=-3)
	8: Level layer 3 (off=-4)
	9: Snake layer 4 (off=-4)
	10: Level layer 4 (off=-5)
	11: Foreground layer (off=0)
*/

class Renderer extends DefaultHandler {
	private Graphics2D canvasGr;
	private BufferedImage canvasImg;
	private BufferStrategy buffStrat;
	private Canvas canvas;
	private HashMap<RenderingHints.Key, Object> rendHints;
	private BufferedImage[] buffImg;
	private Graphics2D[] buffImgGr;
	public BufferedImage[] texture;
	
	private String gameRoot;
	private int numLayers;
	private int textSize;
	private int offSize;
	
	private int picSizeX;
	private int picSizeY;
	
	private String textPrefix;
	
	public Renderer(Canvas c, String textFile) {
		buffImg = new BufferedImage[12];
		buffImgGr = new Graphics2D[12];
		rendHints = new HashMap<RenderingHints.Key, Object>();
		rendHints.put(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_SPEED);
		gameRoot = Globals.GAMEROOT;
		texture = new BufferedImage[2000];
		loadTextures(textFile);
		initRenderer(c);
	}
	
	private void initRenderer(Canvas c) {
		//init Graphics interface to canvas
		canvas = c;
		picSizeX = canvas.getWidth();
		picSizeY = canvas.getHeight();
		
		canvasImg = new BufferedImage(picSizeX, picSizeY,BufferedImage.TYPE_INT_ARGB);
		canvasGr = canvasImg.createGraphics();
		canvasGr.addRenderingHints(rendHints);
		
		//init offscreen layers
		for (int i = 0; i < buffImg.length; i++) {
			buffImg[i] = new BufferedImage(picSizeX,picSizeY,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[i] = buffImg[i].createGraphics();
		}
	}
	
	public void clearScreen(Color col, int x, int y, int w, int h) {
		canvasGr.setColor(col);
		canvasGr.fillRect(x,y,w,h);
	}
	
	public void drawRect(int layer, Color col, int x, int y, int w, int h) {
		buffImgGr[layer].setColor(col);
		buffImgGr[layer].fillRect(x,y,w,h);
	}
	
	public void reNewLayer(int layer) {
		buffImg[layer] = new BufferedImage(picSizeX,picSizeY,BufferedImage.TYPE_INT_ARGB);
		buffImgGr[layer] = buffImg[layer].createGraphics();
	}
	
	public void reNewAllLayers() {
		for (int layer = 0; layer < buffImg.length; layer++) {
			buffImg[layer] = new BufferedImage(picSizeX,picSizeY,BufferedImage.TYPE_INT_ARGB);
			buffImgGr[layer] = buffImg[layer].createGraphics();
		}
	}
	
	public synchronized void drawScreen() {
		preRender();
		flushScreen();
	}
	
	public void flushScreen() {
		try {
			Graphics g = canvas.getGraphics();
			g.drawImage(canvasImg,0,0,canvas);
		} catch (Exception e) {
		}
	}
	
	public void preRender() {
		BufferedImage b;
		for (int i = 0; i < 12; i++) {
			b = buffImg[i];
			canvasGr.drawImage(b, new AffineTransform(), canvas);
		}
	}
	
	public void drawImageOnScreen(int txt, int x, int y) {
		try {
			AffineTransform a = new AffineTransform(1.0, 0.0, 0.0, 1.0, x, y);
			canvasGr.drawImage(texture[txt], a, canvas);
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	public void drawImage(int layer, int txt, int x, int y) {
		AffineTransform a = new AffineTransform(1.0, 0.0, 0.0, 1.0, x*1.0, y*1.0);
		try {
			buffImgGr[layer].drawImage(texture[txt], a, canvas);
		} catch (IndexOutOfBoundsException e) {
			System.out.println("Error: "+e.getMessage());
		} catch (Exception e) {
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	private void loadTextures(String file) {
		try {
			DefaultHandler handler = this;
			SAXParser saxParser = SAXParserFactory.newInstance().newSAXParser();
			saxParser.parse( new File( file ), handler );
		} catch( Throwable t ) {
			t.printStackTrace();
		}
	}
	
	//Code for parsing graphics file
	public void startDocument() throws SAXException {
	}
	public void endDocument() throws SAXException {
	}
	public void characters( char[] buf, int offset, int len ) throws SAXException {
	}
	
	public void startElement( String namespaceURI,
	                          String localName,   // local name
	                          String qName,       // qualified name
	                          Attributes attr )
	                         throws SAXException {
		if (qName.equals("info")) {
			parseInfo(attr);
		} else if (qName.equals("tile")) {
			parseTile(attr);
		}
	}
	
	private void parseInfo(Attributes attr) {
		if (attr.getValue("type").equals("prefix"))
			textPrefix = attr.getValue("content");
	}
	
	private void parseTile(Attributes attr) {
		URL url;
		BufferedImage img;
		int n = Integer.parseInt(attr.getValue("number"));
		try {
			url = new URL(gameRoot+textPrefix+attr.getValue("file"));
	//		System.out.println(n+"  "+url);
			img = ImageIO.read(url);
			texture[n] = img;
		} catch (Exception e) {
			System.out.println("Renderer: \n");
			e.printStackTrace();
		}
	}
}